# Copyright (c) 2022 uidom
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT


from .widgets import *  # isort: skip
from .htmlelement import *  # isort: skip
from .htmldocument import *  # isort: skip
from .xcomponent_js import *  # isort: skip
# from .xshadowcomponent import *  # isort: skip
from .css import *  # isort: skip
from .elements import *  # isort: skip 
from .htmlelement import *  # isort: skip
from .icons import *  # isort: skip
from .jinja import *  # isort: skip
from .links import *  # isort: skip
from .src import *  # isort: skip
# from .tailwindcss import *  # isort: skip
from .vue import *  # isort: skip
from .html_to_dom import *  # isort: skip

# https://github.com/microsoft/vscode/issues/35350#issuecomment-1093627529 
# here is the need to include isort skip.
